@@ -63,8 +63,8 @@ beats rebuilding the kernel!
   1999-03-17
   Linux/SPARC modifications by Jeffrey Connell <ankh@canuck.gen.nz>:
   It seems that the KDKBDREP ioctl is not available on this platform.
-  However, Linux/SPARC has its own ioctl for this, with yet another
-  different measurement system.  Thus, try for KIOCSRATE, too.
+  However, Linux/SPARC has its own ioctl for this (since 2.1.30),
+  with yet another measurement system.  Thus, try for KIOCSRATE, too.
 
 */
 
@@ -77,7 +77,6 @@ beats rebuilding the kernel!
 
 #ifdef __sparc__
 #include <asm/param.h>
-#include <asm/kbio.h>
 #endif
 
 #ifndef KDKBDREP
@@ -85,10 +84,22 @@ beats rebuilding the kernel!
 #define KDKBDREP        0x4B52  /* set keyboard delay/repeat rate;
 				 * actually used values are returned */
 struct kbd_repeat {
-        int delay;      /* in msec; <= 0: don't change */
+        int delay;        /* in msec; <= 0: don't change */
+#ifdef __sparc__
+	int rate;         /* in msec; <= 0: don't change */
+#else
         int period;       /* in msec; <= 0: don't change */
-};
 #endif
+};
+#endif /* KDKBDREP */
+
+#ifdef __sparc__
+#define KBD_PERIOD(var) var.rate
+#else
+#define KBD_PERIOD(var) var.period
+#endif
+
+#include <signal.h>
 
 #include "nls.h"
 #include "version.h"
@@ -104,15 +115,18 @@ static int valid_delays[] = { 250, 500, 750, 1000 };
 
 static int
 KDKBDREP_ioctl_ok(double rate, int delay, int silent) {
+#if defined(KDKBDREP) && !defined(__sparc__)
 	/* This ioctl is defined in <linux/kd.h> but is not
-	   implemented anywhere - must be in some m68k patches. */
+	   implemented anywhere - must be in some m68k patches.
+	   We cannot blindly try unimplemented ioctls on sparc64 -
+	   the 32<->64bit transition layer does not like it. */
 	struct kbd_repeat kbdrep_s;
 
 	/* don't change, just test */
-	kbdrep_s.period = -1;
+	KBD_PERIOD(kbdrep_s) = -1;
 	kbdrep_s.delay = -1;
 	if (ioctl( 0, KDKBDREP, &kbdrep_s )) {
-		if (errno == EINVAL)
+		if (errno == EINVAL || errno == ENOTTY)
 			return 0;
 		perror( "ioctl(KDKBDREP)" );
 		exit( 1 );
@@ -120,16 +134,16 @@ KDKBDREP_ioctl_ok(double rate, int delay, int silent) {
 
 #if 0
 	printf("old delay %d, period %d\n",
-	       kbdrep_s.delay, kbdrep_s.period);
+	       kbdrep_s.delay, KBD_PERIOD(kbdrep_s));
 #endif
 
 	/* do the change */
 	if (rate == 0)				  /* switch repeat off */
-		kbdrep_s.period = 0;
+		KBD_PERIOD(kbdrep_s) = 0;
 	else
-		kbdrep_s.period  = 1000.0 / rate; /* convert cps to msec */
-	if (kbdrep_s.period < 1)
-		kbdrep_s.period = 1;
+		KBD_PERIOD(kbdrep_s)  = 1000.0 / rate; /* convert cps to msec */
+	if (KBD_PERIOD(kbdrep_s) < 1)
+		KBD_PERIOD(kbdrep_s) = 1;
 	kbdrep_s.delay = delay;
 	if (kbdrep_s.delay < 1)
 		kbdrep_s.delay = 1;
@@ -140,16 +154,19 @@ KDKBDREP_ioctl_ok(double rate, int delay, int silent) {
 	}
 
 	/* report */
-	if (kbdrep_s.period == 0)
+	if (KBD_PERIOD(kbdrep_s) == 0)
 		rate = 0;
 	else
-		rate = 1000.0 / (double) kbdrep_s.period;
+		rate = 1000.0 / (double) KBD_PERIOD(kbdrep_s);
 
 	if (!silent)
 		printf( _("Typematic Rate set to %.1f cps (delay = %d ms)\n"),
 			rate, kbdrep_s.delay );
 
 	return 1;			/* success! */
+#else /* no KDKBDREP or __sparc__ */
+	return 0;
+#endif /* KDKBDREP */
 }
 
 static int
@@ -164,10 +181,10 @@ KIOCSRATE_ioctl_ok(double rate, int delay, int silent) {
 		exit( 1 );
 	}
 
-	kbdrate_s.period = (int) (rate + 0.5);  /* round up */
+	KBD_PERIOD(kbdrate_s) = (int) (rate + 0.5);  /* round up */
 	kbdrate_s.delay = delay * HZ / 1000;  /* convert ms to Hz */
-	if (kbdrate_s.period > 50)
-		kbdrate_s.period = 50;
+	if (KBD_PERIOD(kbdrate_s) > 50)
+		KBD_PERIOD(kbdrate_s) = 50;
 
 	if (ioctl( fd, KIOCSRATE, &kbdrate_s )) {
 		perror( "ioctl(KIOCSRATE)" );
@@ -177,7 +194,7 @@ KIOCSRATE_ioctl_ok(double rate, int delay, int silent) {
 
 	if (!silent)
 		printf( "Typematic Rate set to %d cps (delay = %d ms)\n",
-			kbdrate_s.period, kbdrate_s.delay * 1000 / HZ );
+			KBD_PERIOD(kbdrate_s), kbdrate_s.delay * 1000 / HZ );
 
 	return 1;
 #else /* no KIOCSRATE */
@@ -185,10 +202,16 @@ KIOCSRATE_ioctl_ok(double rate, int delay, int silent) {
 #endif /* KIOCSRATE */
 }
 
+static void
+sigalrmhandler( int sig ) {
+	fprintf( stderr, "kbdrate: Failed waiting for kbd controller!\n" );
+	raise( SIGINT );
+}
+
 int
 main( int argc, char **argv ) {
 #ifdef __sparc__
-	double      rate = 5.0;      /* Default rate */
+	double      rate = 20.0;      /* Default rate */
 	int         delay = 200;     /* Default delay */
 #else
 	double      rate = 10.9;     /* Default rate */
@@ -238,8 +261,9 @@ main( int argc, char **argv ) {
 		return 0;
 
 
-	/* The ioport way */
+	/* The ioport way - will crash on sparc */
 
+#ifndef __sparc__
 	for (i = 0; i < RATE_COUNT; i++)
 		if (rate * 10 >= valid_rates[i]) {
 			value &= 0x60;
@@ -260,23 +284,40 @@ main( int argc, char **argv ) {
 		exit( 1 );
 	}
 
+	signal( SIGALRM, sigalrmhandler );
+	alarm( 3 );
+
 	do {
 		lseek( fd, 0x64, 0 );
-		read( fd, &data, 1 );
+		if (read( fd, &data, 1 ) == -1) {
+			perror( "read" );
+			exit( 1 );
+		}
 	} while ((data & 2) == 2 );  /* wait */
 
 	lseek( fd, 0x60, 0 );
 	data = 0xf3;                 /* set typematic rate */
-	write( fd, &data, 1 );
+	if (write( fd, &data, 1 ) == -1) {
+		perror( "write" );
+		exit( 1 );
+	}
 
 	do {
 		lseek( fd, 0x64, 0 );
-		read( fd, &data, 1 );
+		if (read( fd, &data, 1 ) == -1) {
+			perror( "read" );
+			exit( 1 );
+		}
 	} while ((data & 2) == 2 );  /* wait */
 
+	alarm( 0 );
+
 	lseek( fd, 0x60, 0 );
 	sleep( 1 );
-	write( fd, &value, 1 );
+	if (write( fd, &value, 1 ) == -1) {
+		perror( "write" );
+		exit( 1 );
+	}
 
 	close( fd );
 
@@ -285,5 +326,7 @@ main( int argc, char **argv ) {
 			valid_rates[value & 0x1f] / 10.0,
 			valid_delays[ (value & 0x60) >> 5 ] );
 
+#endif
+
 	return 0;
 }
