@@ -10,7 +10,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/kd.h>
+#include <linux/kd.h>
 #include "paths.h"
 #include "findfile.h"
 #include "kdmapop.h"
@@ -32,6 +32,7 @@ static char *mapsuffixes[] = { "", ".trans", "_to_uni.trans", 0 };
 #include "version.h"
 
 int verbose = 0;
+int debug = 0;
 
 int
 main(int argc, char *argv[]) {
@@ -52,7 +53,7 @@ main(int argc, char *argv[]) {
 		argv++;
 	}
 
-	fd = getfd();
+	fd = getfd(NULL);
 
 	if (argc >= 3 && !strcmp(argv[1], "-o")) {
 	    saveoldmap(fd, argv[2]);
@@ -125,7 +126,7 @@ readnewmapfromfile(int fd, char *mfil, char *buf, unsigned short *ubuf) {
 	FILE *fp;
 	struct stat stbuf;
 	int u = 0;
-	int lineno;
+	int lineno = 0;
 
 	if ((fp = findfile(mfil, mapdirpath, mapsuffixes)) == NULL) {
 	        fprintf(stderr, _("mapscrn: cannot open map file _%s_\n"),
@@ -214,18 +215,18 @@ ctoi(unsigned char *s) {
 
 	if ((strncmp(s,"0x",2) == 0) && 
 	    (strspn(s+2,"0123456789abcdefABCDEF") == strlen(s+2)))
-		sscanf(s+2,"%x",&i);
+		(void)sscanf(s+2,"%x",&i);
 
 	else if ((*s == '0') &&
 		 (strspn(s,"01234567") == strlen(s)))
-		sscanf(s,"%o",&i);
+		(void)sscanf(s,"%o",&i);
 
 	else if (strspn(s,"0123456789") == strlen(s)) 
-		sscanf(s,"%d",&i);
+		(void)sscanf(s,"%d",&i);
 
 	else if ((strncmp(s,"U+",2) == 0) && strlen(s) == 6 &&
 		 (strspn(s+2,"0123456789abcdefABCDEF") == 4))
-		sscanf(s+2,"%x",&i);
+		(void)sscanf(s+2,"%x",&i);
 
 	else if ((strlen(s) == 3) && (s[0] == '\'') && (s[2] == '\''))
 		i=s[1];
